#|
  Step/theme.jl

  Copyright (C) 1999 Michele Campeotto <micampe@f2s.com>

  This theme is based on WindowMaker, the GNUStep project window manager.
  I ported it to Sawmill because I liked his look'n'feel but Window Maker
  is'nt working well with Gnome.
  
  Credits go to (in no particular order):
   - John Harper         <john@dcs.warwick.ac.uk> (twm and Sawfish, of course)
   - Kevin D. Knerr, Sr. <ld_barthel@yahoo.com>   (AquaX)
   - Nathan Froyd        <froydnj@rose-hulman.edu>(synergy)
   - Pedro Lopes         <paol@teleweb.pt>        (Operational)
   - Isaac Feitler       <ike@mit.edu>            (blueberry)

   - The Window Maker team        http://www.windowmaker.org
|#

;; --- Frame preferences ---------------------------------------------------

(defgroup Step-frame "Step theme"
    :group appearance)

(defcustom Step:title-alignement 'center
    "Alignement of the text in the titlebar"
    :group (appearance Step-frame)
    :type symbol
    :options (left center right)
    :after-set after-setting-frame-option)

(defcustom Step:flat-buttons nil
    "Use flat buttons on titlebar"
    :group (appearance Step-frame)
    :type boolean
    :after-set after-setting-frame-option)

(defcustom Step:original-style nil
    "Use original (flat) style for pressed buttons"
    :group (appearance Step-frame)
    :type boolean
    :after-set after-setting-frame-option)

(defcustom Step:titlebar-active-color (get-color "#3d70785191ea")
    "Color of the active window titlebar"
    :group (appearance Step-frame)
    :type color
    :after-set after-setting-frame-option)

(defcustom Step:titlebar-inactive-color (get-color "gray75")
    "Color of inactive windows titlebars"
    :group (appearance Step-frame)
    :type color
    :after-set after-setting-frame-option)

(defcustom Step:title-active-color (get-color "white")
    "Color of the active window title"
    :group (appearance Step-frame)
    :type color
    :after-set after-setting-frame-option)

(defcustom Step:title-inactive-color (get-color "gray25")
    "Color of inactive windows titles"
    :group (appearance Step-frame)
    :type color
    :after-set after-setting-frame-option)

(defcustom Step:resizebar-active-color (get-color "gray75")
    "Color of the active window resizebar"
    :group (appearance Step-frame)
    :type color
    :after-set after-setting-frame-option)

(defcustom Step:resizebar-inactive-color (get-color "gray75")
    "Color of inactive windows resizebars"
    :group (appearance Step-frame)
    :type color
    :after-set after-setting-frame-option)

;; --- Colors functions ----------------------------------------------------

(define recolor-image (lambda (img color)
    (when (and (imagep img) (colorp color))
        (define rgb (color-rgb color))
        (define red (nth 0 rgb))
        (define green (nth 1 rgb))
        (define blue (nth 2 rgb))
        (set-image-modifier img 'red
            (list (quotient red 256) 255 (quotient red 256)))
        (set-image-modifier img 'green
            (list (quotient green 256) 255 (quotient green 256)))
        (set-image-modifier img 'blue
            (list (quotient blue 256) 255 (quotient blue 256)))
    )
))

(defun brighten-color (color percentage)
    (let* ((red (car (color-rgb color)))
           (green (cadr (color-rgb color)))
           (blue (car (cddr (color-rgb color))))
           (red-delta (quotient (* red percentage) 100))
           (green-delta (quotient (* green percentage) 100))
           (blue-delta (quotient (* blue percentage) 100)))
        (get-color-rgb (min (+ red-delta red) 65535)
                       (min (+ green-delta green) 65535)
                       (min (+ blue-delta blue) 65535)
        )
    )
)
(defun darken-color (color percentage)
    (let* ((red (car (color-rgb color)))
           (green (cadr (color-rgb color)))
           (blue (car (cddr (color-rgb color))))
           (red-delta (quotient (* red percentage) 100))
           (green-delta (quotient (* green percentage) 100))
           (blue-delta (quotient (* blue percentage) 100)))
        (get-color-rgb (max (- red red-delta) 0)
                       (max (- green green-delta) 0)
                       (max (- blue blue-delta) 0)
        )
    )
)

;; --- Frame definition ----------------------------------------------------

(define title-font
    (get-font "-*-helvetica-bold-r-normal-*-12-*-*-*-*-*-*"))
(define title-colors
    (lambda () `(
        (inactive . ,Step:title-inactive-color)
        (focused . ,Step:title-active-color)
    ))
)
(define text-align
    (lambda ()
        Step:title-alignement
    )
)

(define titlebar-colors
    (lambda () `(
        (inactive . ,Step:titlebar-inactive-color)
        (focused . ,Step:titlebar-active-color)
    ))
)
(define resizebar-colors
    (lambda () `(
        (inactive . ,Step:resizebar-inactive-color)
        (focused . ,Step:resizebar-active-color)
;        (highlighted . ,(brighten-color Step:resizebar-active-color 15))
    ))
)
(define border-color
    (get-color "black"))

(define bevel-up-left-color
    (lambda () `(
        (inactive . ,(brighten-color Step:titlebar-inactive-color 50))
        (focused . ,(brighten-color Step:titlebar-active-color 50))
    ))
)
(define bevel-down-right-color
    (lambda () `(
        (inactive . ,(darken-color Step:titlebar-inactive-color 50))
        (focused . ,(darken-color Step:titlebar-active-color 50))
    ))
)
(define bevel-resize-up-left-color
    (lambda () `(
        (inactive . ,(brighten-color Step:resizebar-inactive-color 50))
        (focused . ,(brighten-color Step:resizebar-active-color 50))
    ))
)
(define bevel-resize-down-right-color
    (lambda () `(
        (inactive . ,(darken-color Step:resizebar-inactive-color 50))
        (focused . ,(darken-color Step:resizebar-active-color 50))
    ))
)

;; --- Frame parts ---------------------------------------------------------

(define minimize-bevel-original `(
    (inactive . ,(make-image "minimize-bevel-button.png"))
    (focused . ,(make-image "minimize-bevel-button.png"))
    (clicked . ,(make-image "minimize-bevel-pressed.png"))
))
(define minimize-bevel-new `(
    (inactive . ,(make-image "minimize-bevel-button.png"))
    (focused . ,(make-image "minimize-bevel-button.png"))
    (clicked . ,(make-image "minimize-bevel-pressed-new.png"))
))

(define minimize-flat `(
    (inactive . ,(recolor-image (make-image "minimize-flat-button.png")
                  Step:title-inactive-color))
    (focused . ,(recolor-image (make-image "minimize-flat-button.png")
                 Step:title-active-color))
    (clicked . ,(make-image "minimize-flat-pressed.png"))
))

(define minimize
    (lambda ()
        (if Step:flat-buttons
            minimize-flat
            (if Step:original-style
                minimize-bevel-original
                minimize-bevel-new
            )
        )
    )
)

(define close-bevel-original `(
    (inactive . ,(make-image "close-bevel-button.png"))
    (focused . ,(make-image "close-bevel-button.png"))
    (clicked . ,(make-image "close-bevel-pressed.png"))
))
(define close-bevel-new `(
    (inactive . ,(make-image "close-bevel-button.png"))
    (focused . ,(make-image "close-bevel-button.png"))
    (clicked . ,(make-image "close-bevel-pressed-new.png"))
))

(define close-flat `(
    (inactive . ,(recolor-image (make-image "close-flat-button.png")
                  Step:title-inactive-color))
    (focused . ,(recolor-image (make-image "close-flat-button.png")
                 Step:title-active-color))
    (clicked . ,(make-image "close-flat-pressed.png"))
))

(define close
    (lambda ()
        (if Step:flat-buttons
            close-flat
            (if Step:original-style
                close-bevel-original
                close-bevel-new
            )
        )
    )
)


(define titlebar `(
; Black border
    ((background . ,border-color)
     (right-edge . 20)
     (left-edge . 20)
     (top-edge . -23)
     (height . 1)
     (class . top-border))
    ((background . ,border-color)
     (right-edge . 20)
     (left-edge . 20)
     (top-edge . -1)
     (height . 1))
; Beveled border
    ((background . ,bevel-up-left-color)
     (right-edge . 20)
     (left-edge . 20)
     (top-edge . -22)
     (height . 1)
     (class . top-border))
    ((background . ,bevel-down-right-color)
     (right-edge . 20)
     (left-edge . 20)
     (top-edge . -2)
     (height . 1))
; Titlebar
    ((background . ,titlebar-colors)
     (left-edge . 20)
     (right-edge . 20)
     (top-edge . -21)
     (height . 19)
     (cursor . left_ptr)
     (class . title))
    ((background . ,titlebar-colors)
     (foreground . ,title-colors)
     (font . ,title-font)
     (text . ,window-name)
     (x-justify . ,text-align)
     (y-justify . center)
     (left-edge . 26)
     (right-edge . 26)
     (top-edge . -21)
     (height . 19)
     (cursor . left_ptr)
     (class . title))
))

(define titlebar-transient `(
; Black Border
    ((background . ,border-color)
     (left-edge . -1)
     (top-edge . -22)
     (width . 1)
     (height . 21)
     (class . top-left-corner))
    ((background . ,border-color)
     (right-edge . 20)
     (left-edge . -1)
     (top-edge . -23)
     (height . 1)
     (class . top-border))
    ((background . ,border-color)
     (right-edge . 20)
     (left-edge . -1)
     (top-edge . -1)
     (height . 1))
; Bevel border
    ((background . ,bevel-up-left-color)
     (left-edge . 0)
     (top-edge . -22)
     (width . 1)
     (height . 21)
     (class . top-left-corner))
    ((background . ,bevel-up-left-color)
     (right-edge . 20)
     (left-edge . 0)
     (top-edge . -22)
     (height . 1)
     (class . top-border))
    ((background . ,bevel-down-right-color)
     (right-edge . 20)
     (left-edge . 1)
     (top-edge . -2)
     (height . 1))
; Titlebar
    ((background . ,titlebar-colors)
     (foreground . ,title-colors)
     (font . ,title-font)
     (text . ,window-name)
     (x-justify . ,text-align)
     (y-justify . center)
     (left-edge . 1)
     (right-edge . 20)
     (top-edge . -21)
     (height . 19)
     (cursor . left_ptr)
     (class . title))
))

(define button-minimize `( 
; Black border
    ((background . ,border-color)
     (left-edge . -1)
     (width . 21)
     (height . 1)
     (top-edge . -23)
     (class . top-left-corner))
    ((background . ,border-color)
     (left-edge . 21)
     (width . ,(lambda () (if Step:flat-buttons 1 0)))
     (top-edge . -22)
     (height . 21))
    ((background . ,border-color)
     (left-edge . -1)
     (width . 21)
     (height . 1)
     (top-edge . -1))
    ((background . ,border-color)
     (left-edge . -1)
     (width . 1)
     (top-edge . -22)
     (height . 21)
     (class . top-left-corner))
; Bevel border
    ((background . ,bevel-up-left-color)
     (left-edge . 0)
     (width . 21)
     (height . 1)
     (top-edge . -22)
     (class . top-left-corner))
    ((background . ,bevel-down-right-color)
     (left-edge . 20)
     (width . ,(lambda () (if Step:flat-buttons 1 0)))
     (top-edge . -21)
     (height . 21))
    ((background . ,bevel-down-right-color)
     (left-edge . 0)
     (width . 20)
     (height . 1)
     (top-edge . -2))
    ((background . ,bevel-up-left-color)
     (left-edge . 0)
     (width . 1)
     (top-edge . -21)
     (height . 20)
     (class . top-left-corner))

    ((background . ,bevel-up-left-color)
     (left-edge . 22)
     (width . ,(lambda () (if Step:flat-buttons 1 0)))
     (top-edge . -22)
     (height . 21))
; Button icon
    ((background . ,titlebar-colors)
     (foreground . ,minimize)
     (left-edge . 1)
     (width . 19)
     (top-edge . -21)
     (height . 19)
     (cursor . left_ptr)
     (class . iconify-button))
))

(define button-close `(
; Black border
    ((background . ,border-color)
     (right-edge . -1)
     (width . 21)
     (height . 1)
     (top-edge . -23)
     (class . top-right-corner))
    ((background . ,border-color)
     (right-edge . 21)
     (width . ,(lambda () (if Step:flat-buttons 1 0)))
     (top-edge . -22)
     (height . 21))
    ((background . ,border-color)
     (right-edge . -1)
     (width . 21)
     (height . 1)
     (top-edge . -1))
    ((background . ,border-color)
     (right-edge . -1)
     (width . 1)
     (top-edge . -22)
     (height . 21)
     (class . top-right-corner))
; Bevel border
    ((background . ,bevel-up-left-color)
     (right-edge . 0)
     (width . 20)
     (height . 1)
     (top-edge . -22)
     (class . top-right-corner))
    ((background . ,bevel-up-left-color)
     (right-edge . 20)
     (width . ,(lambda () (if Step:flat-buttons 1 0)))
     (top-edge . -22)
     (height . 21))
    ((background . ,bevel-down-right-color)
     (right-edge . 0)
     (width . 20)
     (height . 1)
     (top-edge . -2))
    ((background . ,bevel-down-right-color)
     (right-edge . 0)
     (width . 1)
     (top-edge . -21)
     (height . 20)
     (class . top-right-corner))

    ((background . ,bevel-down-right-color)
     (right-edge . 22)
     (width . ,(lambda () (if Step:flat-buttons 1 0)))
     (top-edge . -21)
     (height . 20))
; Button icon
    ((background . ,titlebar-colors)
     (foreground . ,close)
     (right-edge . 1)
     (width . 19)
     (top-edge . -21)
     (height . 19)
     (cursor . left_ptr)
     (class . close-button))
))

(define resizebar `(
;; Bottom left corner
    ((background . ,border-color)
     (left-edge . -1)
     (width . 1)
     (bottom-edge . -9)
     (height . 9)
     (class . left-border))
    ((background . ,bevel-resize-down-right-color)
     (left-edge . 29)
     (width . 1)
     (bottom-edge . -8)
     (height . 6))
    ((background . ,bevel-resize-up-left-color)
     (left-edge . 30)
     (width . 1)
     (bottom-edge . -8)
     (height . 6))
    ((background . ,resizebar-colors)
     (left-edge . 0)
     (width . 29)
     (bottom-edge . -8)
     (height . 6)
     (class . bottom-left-corner))
;; Bottom border
    ((background . ,bevel-resize-down-right-color)
     (right-edge . 0)
     (left-edge . 0)
     (bottom-edge . -1)
     (height . 1))
    ((background . ,bevel-resize-up-left-color)
     (right-edge . 0)
     (left-edge . 0)
     (bottom-edge . -2)
     (height . 1))
    ((background . ,border-color)
     (right-edge . -1)
     (left-edge . -1)
     (bottom-edge . -9)
     (height . 1))
    ((background . ,resizebar-colors)
     (left-edge . 31)
     (right-edge . 31)
     (bottom-edge . -8)
     (height . 6)
     (class . bottom-border))
;; Bottom right corner
    ((background . ,border-color)
     (right-edge . -1)
     (width . 1)
     (bottom-edge . -9)
     (height . 9)
     (class . left-border))
    ((background . ,bevel-resize-down-right-color)
     (right-edge . 30)
     (width . 1)
     (bottom-edge . -8)
     (height . 6))
    ((background . ,bevel-resize-up-left-color)
     (right-edge . 29)
     (width . 1)
     (bottom-edge . -8)
     (height . 6))
    ((background . ,resizebar-colors)
     (right-edge . 0)
     (width . 29)
     (bottom-edge . -8)
     (height . 6)
     (class . bottom-right-corner))
))

(define window-border `(
    ((background . ,border-color)
     (right-edge . -1)
     (width . 1)
     (top-edge . 0)
     (bottom-edge . -1)
     (class . right-border))
    ((background . ,border-color)
     (left-edge . -1)
     (width . 1)
     (top-edge . 0)
     (bottom-edge . -1)
     (class . left-border))
    ((background . ,border-color)
     (left-edge . 0)
     (right-edge . 0)
     (bottom-edge . -1)
     (height . 1)
     (class . bottom-border))
))

;; --- Frame definition -------------------

(define normal-frame nil)
(define shaped-frame nil)
(define transient-frame nil)
(define shaped-transient-frame nil)

(define create-frames
    (lambda ()
        (setq normal-frame (append
            titlebar
            window-border
            button-close
            button-minimize
            resizebar
        ))
        (setq shaped-frame (append
            titlebar
            button-close
            button-minimize
        ))
        (setq transient-frame (append
            titlebar-transient
            window-border
            button-close
        ))
        (setq shaped-transient-frame (append
            titlebar-transient
            button-close
        ))
    )
)

(create-frames)

(add-frame-style 'Step
    (lambda (w type)
        (case type
            ((default) normal-frame)
            ((shaped) shaped-frame)
            ((transient) transient-frame)
            ((shaped-transient) shaped-transient-frame)
        )
    )
)
